@@ -11,79 +11,6 @@
 ##    
 ##########################################################################
 
-####################
-findInertInputs<-function(object,threshold=0.1){
-  P_hat=object@p*object@beta_hat*object@CL/sum(object@beta_hat*object@CL)
-  index_inert=which(P_hat<threshold)
-  
-  cat('The estimated normalized inverse range parameters are :', P_hat,'\n')
-  if(length(which(P_hat<0.1))>0){
-    cat('The inputs ', index_inert, 'are suspected to be inert inputs','\n')
-  }else{
-    cat('no input is suspected to be an inert input', '\n')
-  }
-  P_hat
-}
-
-####################
-neg_log_marginal_post_approx_ref <- function(param,nugget, nugget.est,R0,X,zero_mean,output,CL,a,b,kernel_type,alpha) {
-   #####this has mean X, we should also include the case where X is not zero
-   #####
-   lml=log_marginal_lik(param,nugget,nugget.est,R0,X,zero_mean,output,kernel_type,alpha);
-   lp=log_approx_ref_prior(param,nugget,nugget.est,CL,a,b);
-     
-  -(lml+lp)
-
-}
-
-####################
-neg_log_marginal_post_ref<- function(param,nugget, nugget.est,R0,X,zero_mean,output,prior_choice,kernel_type,alpha) {
-  
-  lmp=log_ref_marginal_post(param,nugget,nugget.est,R0,X,zero_mean,output,kernel_type,alpha);
-  
-  
-  if(prior_choice=='ref_xi'){
-     if(nugget.est==T){###note that in this case nu is also needed to be transformed have tail properties
-      #-sum(param)-lmp  ###this will let nu be non zero so we need to be careful
-       -sum(param[1:(length(param))])-lmp ###this might be fine when intercept is in the trend
-     }else{ ###no nugget
-      -sum(param)-lmp
-     }
-  }else if(prior_choice=='ref_gamma'){
-    if(nugget.est==T){###note that in this case nu is also needed to be transformed have tail properties
-      -2*sum(param[1:(length(param)-1)])-lmp 
-    }else{ ###no nugget
-      -2*sum(param)-lmp
-    }
-  }
-}
-
-
-
-########################
-neg_log_marginal_post_approx_ref_deriv<- function(param,nugget,nugget.est,R0,X,zero_mean,output,CL,a,b,kernel_type,alpha) {
-  lml_dev=log_marginal_lik_deriv(param,nugget,nugget.est,R0,X,zero_mean,output,kernel_type,alpha)
-  lp_dev=log_approx_ref_prior_deriv(param,nugget,nugget.est,CL,a,b)
-  
-  -(lml_dev+lp_dev)*exp(param)
-    
-}
-#############this is a function to search the lower bounds for range parameters beta
-#####need R0 in the function
-search_LB_prob<-function(param, R0,COND_NUM_UB,p,kernel_type,alpha,nugget){
-  num_obs=dim(R0[[1]])[1]
-  propose_prob=exp(param)/(exp(param)+1)
-  LB=NULL
-  for( i_LB in 1:p){
-    LB=c(LB, log(-log(propose_prob)/(max(R0[[i_LB]]))))    ###LB is log beta
-  }
-  
-  R=separable_kernel(R0,exp(LB),kernel_type,alpha)  
-  
-  R=as.matrix(R)
-  R=R+nugget*diag(num_obs)
-  (kappa(R)-COND_NUM_UB)^2
-}
 ###############1 dim hig02 function
 hig02 <- function(s)
 {
@@ -197,8 +124,8 @@ detpep10curv <- function(xx){
   term1 <- 4 * (x1 - 2 + 8*x2 - 8*x2^2)^2
   term2 <- (3 - 4*x2)^2
   term3 <- 16 * sqrt(x3+1) * (2*x3-1)^2
-   y <- term1 + term2 + term3               ###no noise
- # y <- term1 + term2 + term3+rnorm(1,0,.5) ###noisy one
+  y <- term1 + term2 + term3               ###no noise
+  # y <- term1 + term2 + term3+rnorm(1,0,.5) ###noisy one
   return(y)
 }
 
@@ -305,3 +232,129 @@ fried <- function(xx)
   y <- term1 + term2 + term3 + term4
   return(y)
 }
+
+####################
+findInertInputs<-function(object,threshold=0.1){
+  P_hat=object@p*object@beta_hat*object@CL/sum(object@beta_hat*object@CL)
+  index_inert=which(P_hat<threshold)
+  
+  cat('The estimated normalized inverse range parameters are :', P_hat,'\n')
+  if(length(which(P_hat<0.1))>0){
+    cat('The inputs ', index_inert, 'are suspected to be inert inputs','\n')
+  }else{
+    cat('no input is suspected to be an inert input', '\n')
+  }
+  P_hat
+}
+
+####################
+neg_log_marginal_post_approx_ref <- function(param,nugget, nugget.est,R0,X,zero_mean,output,CL,a,b,kernel_type,alpha) {
+   #####this has mean X, we should also include the case where X is not zero
+   #####
+   lml=log_marginal_lik(param,nugget,nugget.est,R0,X,zero_mean,output,kernel_type,alpha);
+   lp=log_approx_ref_prior(param,nugget,nugget.est,CL,a,b);
+     
+  -(lml+lp)
+
+}
+
+####################
+neg_log_marginal_post_ref<- function(param,nugget, nugget.est,R0,X,zero_mean,output,prior_choice,kernel_type,alpha) {
+  
+  lmp=log_ref_marginal_post(param,nugget,nugget.est,R0,X,zero_mean,output,kernel_type,alpha);
+  
+  
+  if(prior_choice=='ref_xi'){
+     if(nugget.est==T){###note that in this case nu is also needed to be transformed have tail properties
+      #-sum(param)-lmp  ###this will let nu be non zero so we need to be careful
+       -sum(param[1:(length(param))])-lmp ###this might be fine when intercept is in the trend
+     }else{ ###no nugget
+      -sum(param)-lmp
+     }
+  }else if(prior_choice=='ref_gamma'){
+    if(nugget.est==T){###note that in this case nu is also needed to be transformed have tail properties
+      -2*sum(param[1:(length(param)-1)])-lmp 
+    }else{ ###no nugget
+      -2*sum(param)-lmp
+    }
+  }
+}
+
+
+
+########################
+neg_log_marginal_post_approx_ref_deriv<- function(param,nugget,nugget.est,R0,X,zero_mean,output,CL,a,b,kernel_type,alpha) {
+  lml_dev=log_marginal_lik_deriv(param,nugget,nugget.est,R0,X,zero_mean,output,kernel_type,alpha)
+  lp_dev=log_approx_ref_prior_deriv(param,nugget,nugget.est,CL,a,b)
+  
+  -(lml_dev+lp_dev)*exp(param)
+    
+}
+#############this is a function to search the lower bounds for range parameters beta
+#####need R0 in the function
+search_LB_prob<-function(param, R0,COND_NUM_UB,p,kernel_type,alpha,nugget){
+  num_obs=dim(R0[[1]])[1]
+  propose_prob=exp(param)/(exp(param)+1)
+  LB=NULL
+  for( i_LB in 1:p){
+    LB=c(LB, log(-log(propose_prob)/(max(R0[[i_LB]]))))    ###LB is log beta
+  }
+  
+  R=separable_kernel(R0,exp(LB),kernel_type,alpha)  
+  
+  R=as.matrix(R)
+  R=R+nugget*diag(num_obs)
+  (kappa(R)-COND_NUM_UB)^2
+}
+
+###leave one out 
+leave_one_out_rgasp<-function(object){
+  R_tilde=object@L%*%t(object@L)+object@nugget
+  sigma_2=rep(0,object@num_obs)
+  mean=rep(0,object@num_obs)
+  if(object@zero_mean=="Yes"){
+    for(i in 1:object@num_obs){
+      r_sub=R_tilde[-i,i]
+      L_sub=t(chol(R_tilde[-i,-i]))
+      
+      r_sub_t_R_sub_inv=t(backsolve(t(L_sub),forwardsolve(L_sub,r_sub)))
+      
+      R_sub_inv_y=backsolve(t(L_sub),forwardsolve(L_sub,object@output[-i] ))
+      mean[i]=r_sub_t_R_sub_inv%*%object@output[-i]
+      sigma_2_hat=object@output[-i]%*%R_sub_inv_y/(object@num_obs-1)
+      sigma_2[i]=sigma_2_hat*(R_tilde[i,i]-r_sub_t_R_sub_inv%*%r_sub)
+    }
+  }else{
+    for(i in 1:object@num_obs){
+      r_sub=R_tilde[-i,i]
+      L_sub=t(chol(R_tilde[-i,-i]))
+      r_sub_t_R_sub_inv=t(backsolve(t(L_sub),forwardsolve(L_sub,r_sub)))
+      
+      R_inv_X=backsolve(t(L_sub),forwardsolve(L_sub,(object@X[-i,]) ))
+
+        
+      L_X=t(chol(t(object@X[-i,])%*%R_inv_X))
+      theta_hat=backsolve(t(L_X),forwardsolve(L_X,t(R_inv_X)%*%object@output[-i]))
+
+      tilde_output=object@output[-i]-object@X[-i,]%*%theta_hat
+      mean[i]=object@X[i,]%*%theta_hat+r_sub_t_R_sub_inv%*%tilde_output
+      
+      
+      sigma2_hat=t(tilde_output)%*%backsolve(t(L_sub),forwardsolve(L_sub,tilde_output ))/(object@num_obs-1-object@q)
+      
+      c_star=(R_tilde[i,i]-r_sub_t_R_sub_inv%*%r_sub)
+      
+
+      h_hat=object@X[i,]-t(object@X[-i,])%*%t(r_sub_t_R_sub_inv)
+      c_star_star= c_star+ t(h_hat)%*%backsolve(t(L_X),forwardsolve(L_X,h_hat))
+      sigma_2[i]=sigma2_hat*(c_star_star)
+      #sigma_2[i]=object@sigma2_hat*(c_star)
+      
+    }
+  }
+  
+  return(list(mean=mean, sd=sqrt(sigma_2)))
+  
+  #plot((output-mean)/sqrt(sigma_2))
+
+}
\ No newline at end of file
